<?php
session_start();

define('USERS_FILE', 'data/users.txt');
define('ATTENDANCE_FILE', 'data/attendance.txt');
define('LEAVE_FILE', 'data/leave_requests.txt');
date_default_timezone_set('Asia/Kolkata');

function authenticateUser($username, $password) {
    if (!file_exists(USERS_FILE)) return false;
    foreach (file(USERS_FILE, FILE_IGNORE_NEW_LINES) as $line) {
        $parts = explode('|', $line);
        if (count($parts) < 11) continue;

        list($empId, $name, $designation, $dob, $doj, $salary, $user, $hash, $login, $logout, $break) = $parts;

        if ($user === $username && password_verify($password, $hash)) {
            return [
                'empId' => $empId,
                'name' => $name,
                'login' => $login,
                'logout' => $logout,
                'salary' => $salary
            ];
        }
    }
    return false;
}

// --- Logout ---
if (isset($_GET['logout'])) {
    session_destroy();
    header("Location: index.php");
    exit;
}

// --- Handle Login ---
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['username'], $_POST['password'])) {
    $auth = authenticateUser($_POST['username'], $_POST['password']);
    if ($auth) {
      $_SESSION['authenticated'] = true;
$_SESSION['user'] = $auth['name']; // <-- this is what shows in "Hello, X"
$_SESSION['emp_id'] = $auth['empId'];
$_SESSION['login_time'] = $auth['login'];
$_SESSION['logout_time'] = $auth['logout'];
$_SESSION['salary'] = $auth['salary'];
       
    } else {
        $error = "Invalid credentials";
    }
}

// --- Show Login Form ---
if (!isset($_SESSION['authenticated'])) {
?>
<!DOCTYPE html>
<html>
<head>
    <title>Login - Attendance System</title>
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">
    <style>
        .login-container {
            max-width: 400px;
            margin: 100px auto;
            padding: 20px;
            border: 1px solid #ddd;
            border-radius: 8px;
            background: #fff;
            text-align: center;
        }
        .login-logo {
            max-height: 60px;
            margin-bottom: 20px;
        }
    </style>
</head>
<body class="bg-light">
    <div class="login-container">
        <img src="https://tapvera.com.au/assets/img/logo/black-logo.png" class="login-logo" alt="Logo">
        <h4 class="mb-4">Employee Login</h4>
        <?php if (!empty($error)): ?>
            <div class="alert alert-danger"><?php echo $error; ?></div>
        <?php endif; ?>
        <form method="POST">
            <input type="text" name="username" class="form-control mb-3" placeholder="Username" required>
            <input type="password" name="password" class="form-control mb-3" placeholder="Password" required>
            <button type="submit" class="btn btn-primary w-100">Login</button>
        </form>
    </div>
</body>
</html>
<?php
exit;
}

// --- Logged In View ---
$user = $_SESSION['user'];
$standardLoginTime = $_SESSION['login_time'];
$salary = $_SESSION['salary'];
$records = $logs = [];
$lastIn = $lastOut = null;

// --- Load Attendance Records ---
if (file_exists(ATTENDANCE_FILE)) {
    foreach (file(ATTENDANCE_FILE, FILE_IGNORE_NEW_LINES) as $line) {
        list($date, $u, $action, $type, $time) = explode('|', $line);
        if ($u === $user) {
            $records[] = compact('date', 'action', 'type', 'time');
            if ($action === 'Punch In') $lastIn = "$date $time";
            if ($action === 'Punch Out') $lastOut = "$date $time";
        }
    }
}

// --- Handle Attendance Action ---
if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['action'])) {
    $act = $_POST['action'];
    $time = date('H:i');
    $date = date('Y-m-d');
    $now = strtotime("$date $time");

    if ($act === 'Punch In' && $lastIn && ($now - strtotime($lastIn)) < 15 * 3600) {
        echo "<script>alert('You can Punch In again after 15 hours'); location.href='index.php';</script>";
        exit;
    }

    if ($act === 'Punch Out' && $lastOut && ($now - strtotime($lastOut)) < 5 * 3600) {
        echo "<script>alert('You can Punch Out again after 5 hours'); location.href='index.php';</script>";
        exit;
    }

    $entry = "$date|$user|$act|" . (($act === 'Break Start' || $act === 'Break End') ? 'Break' : '') . "|$time";
    file_put_contents(ATTENDANCE_FILE, $entry . "\n", FILE_APPEND);
    echo "<script>alert('Action recorded successfully!'); location.href='index.php';</script>";
    exit;
}




// --- Date Filter ---
$fromDate = $_GET['from_date'] ?? null;
$toDate = $_GET['to_date'] ?? null;
if ($fromDate && $toDate) {
    $records = array_filter($records, fn($r) => $r['date'] >= $fromDate && $r['date'] <= $toDate);
}

$logs = [];
if ($fromDate && $toDate) {
    // If filtering by date range
    foreach ($records as $rec) {
        $logs[$rec['date']][] = [
            'action' => $rec['action'] . ($rec['type'] ? " ({$rec['type']})" : ''),
            'time' => $rec['time']
        ];
    }
} else {
    // Default to today only
    $today = date('Y-m-d');
    foreach ($records as $rec) {
        if ($rec['date'] === $today) {
            $logs[$rec['date']][] = [
                'action' => $rec['action'] . ($rec['type'] ? " ({$rec['type']})" : ''),
                'time' => $rec['time']
            ];
        }
    }
}

// --- Calculate Salary ---
$daysInMonth = date('t');
$workingDays = count(array_filter($records, fn($r) => $r['action'] === 'Punch In'));
$leaveDays = count(array_filter($records, fn($r) => $r['action'] === 'Leave'));
$absentDays = $daysInMonth - $workingDays - $leaveDays;
$dailyWage = $salary / $daysInMonth;
$deductions = $absentDays * $dailyWage;
$calculatedSalary = $salary - $deductions;
?>


<!DOCTYPE html>
<html>
<head>
    <title>Dashboard</title>
    <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.3.0/dist/css/bootstrap.min.css" rel="stylesheet">
    <style>
        .break-details { display: none; }
        .show-break-details { cursor: pointer; color: blue; text-decoration: underline; }
    </style>
</head>
<body class="container mt-4">
    <div class="d-flex justify-content-between align-items-center">
        <img src="https://tapvera.com.au/assets/img/logo/black-logo.png" height="50">
        <h3>Hello, <?php echo $user; ?></h3>
        <a href="?logout=true" class="btn btn-warning">Logout</a>
    </div>
    <hr>

    <!-- Attendance Buttons -->
    <form method="POST" class="mb-3">
        <button type="submit" name="action" value="Punch In" class="btn btn-success">Punch In</button>
        <button type="submit" name="action" value="Punch Out" class="btn btn-danger">Punch Out</button>
        <button type="submit" name="action" value="Break Start" class="btn btn-info">Start Break</button>
        <button type="submit" name="action" value="Break End" class="btn btn-warning">End Break</button>
    </form>

<!-- Filter Form -->
<form method="GET" class="row g-3 mb-3">
    <div class="col-md-3">
        <label>From Date</label>
        <input type="date" name="from_date" class="form-control" value="<?php echo $fromDate ?? ''; ?>">
    </div>
    <div class="col-md-3">
        <label>To Date</label>
        <input type="date" name="to_date" class="form-control" value="<?php echo $toDate ?? ''; ?>">
    </div>
    <div class="col-md-3 align-self-end">
        <button type="submit" class="btn btn-primary">Filter</button>
        <a href="index.php" class="btn btn-secondary">Reset</a>
        
    <button type="button" class="btn btn-success" onclick="downloadCSV()">Download CSV</button>

    </div>
</form>

<!-- Add a button for downloading the CSV file -->

<br>
<script>
function downloadCSV() {
    const fromDate = document.querySelector('[name="from_date"]').value;
    const toDate = document.querySelector('[name="to_date"]').value;

    // Fetch filtered records (you can implement an AJAX request here if needed)
    const records = <?php echo json_encode($logs); ?>;

    // If no date range is selected, or records are empty, return
    if (!fromDate || !toDate || Object.keys(records).length === 0) {
        alert('Please select a date range to download the attendance.');
        return;
    }

    // Filter the records by date range
    const filteredRecords = {};
    for (let date in records) {
        if (date >= fromDate && date <= toDate) {
            filteredRecords[date] = records[date];
        }
    }

    if (Object.keys(filteredRecords).length === 0) {
        alert('No records found for the selected date range.');
        return;
    }

    // Generate CSV
    let csvContent = "Date,Punch In,Punch Out,Status,Breaks\n";

    for (let date in filteredRecords) {
        filteredRecords[date].forEach(entry => {
            const punchIn = entry['action'] === 'Punch In' ? entry['time'] : '';
            const punchOut = entry['action'] === 'Punch Out' ? entry['time'] : '';
            const status = (entry['action'] === 'Punch In' && new Date(entry['time']) > new Date("<?php echo $standardLoginTime; ?>")) ? 'Late' : 'On Time';
            const breaks = entry['action'].startsWith('Break') ? `${entry['action']} (${entry['time']})` : '';
            csvContent += `${date},${punchIn},${punchOut},${status},${breaks}\n`;
        });
    }

    // Create a downloadable link for the CSV file
    const blob = new Blob([csvContent], { type: 'text/csv' });
    const url = URL.createObjectURL(blob);
    const a = document.createElement('a');
    a.setAttribute('href', url);
    a.setAttribute('download', 'attendance.csv');
    a.click();
}
</script>


    <?php if ($fromDate && $toDate && empty($logs)): ?>
        <div class="alert alert-warning">No records found for the selected date range.</div>
    <?php endif; ?>

    <!-- Attendance Log Table -->
    <table class="table table-bordered">
        <thead><tr><th>Date</th><th>Punch In</th><th>Punch Out</th><th>Status</th><th>Breaks</th></tr></thead>
        <tbody>
        <?php
        foreach ($logs as $date => $entries) {
            $punchIn = $punchOut = $status = null;
            $breakDetails = [];

            foreach ($entries as $e) {
                if ($e['action'] === 'Punch In') {
                    $punchIn = $e['time'];
                    $status = strtotime($punchIn) > strtotime($standardLoginTime) ? 'Late' : 'On Time';
                }
                if ($e['action'] === 'Punch Out') $punchOut = $e['time'];
                if (str_starts_with($e['action'], 'Break')) {
                    $type = strpos($e['action'], 'Start') !== false ? 'start' : 'end';
                    $breakDetails[] = ['type' => $type, 'time' => $e['time']];
                }
            }

            echo "<tr>
                <td>$date</td>
                <td>$punchIn</td>
                <td>$punchOut</td>
                <td>$status</td>
                <td><span class='show-break-details' onclick='toggleBreakDetails(\"$date\")'>View</span></td>
            </tr>
          <tr id='break-details-$date' class='break-details'><td colspan='5'>
                <ul>";
               for ($i = 0; $i < count($breakDetails); $i += 2) {
    // Get the start and end times
    $start = $breakDetails[$i]['time'];
    $end = isset($breakDetails[$i+1]['time']) ? $breakDetails[$i+1]['time'] : 'Not ended';

    // Calculate the break duration if the end time exists
    if ($end !== 'Not ended') {
        $duration = round((strtotime($end) - strtotime($start)) / 60);  // Duration in minutes

        // Check if the break duration exceeds the max limit (75 minutes)
        if ($duration > 75) {
            echo "<li><strong>Warning:</strong> Break from $start to $end exceeded the maximum duration of 1 hour 15 minutes! (Duration: $duration min)</li>";
        } else {
            echo "<li>From $start to $end (Duration: $duration min)</li>";
        }
    } else {
        // Handle breaks that haven't ended yet
        echo "<li>From $start to $end (Duration: Incomplete)</li>";
    }
}
                echo "</ul>
            </td></tr>";
        }
        ?>
        </tbody>
    </table>

    <!-- Salary Info -->
    <h5>Your Monthly Salary: ₹<?php echo number_format($calculatedSalary, 2); ?></h5>

    <!-- Leave Request Form -->
    <h5 class="mt-4">Request Leave</h5>
    <form method="POST" class="row g-3 mb-4">
        <div class="col-md-3">
            <label>From</label>
            <input type="date" name="leave_from" class="form-control" required>
        </div>
        <div class="col-md-3">
            <label>To</label>
            <input type="date" name="leave_to" class="form-control" required>
        </div>
        <div class="col-md-6">
            <label>Reason</label>
            <textarea name="leave_reason" class="form-control" required></textarea>
        </div>
        <div class="col-md-12">
            <button type="submit" class="btn btn-primary w-100">Submit Request</button>
        </div>
    </form>

    <?php
    if ($_SERVER['REQUEST_METHOD'] === 'POST' && isset($_POST['leave_from'], $_POST['leave_to'], $_POST['leave_reason'])) {
        $leave = "$user|{$_POST['leave_from']}|{$_POST['leave_to']}|" . trim($_POST['leave_reason']) . "|Pending";
        file_put_contents(LEAVE_FILE, $leave . "\n", FILE_APPEND);
        echo "<script>alert('Leave request submitted'); location.href='index.php';</script>";
        exit;
    }
    ?>

    <!-- Leave Records -->
    <h5 class="mt-4">Leave Records</h5>
    <table class="table table-bordered">
        <thead><tr><th>From</th><th>To</th><th>Reason</th><th>Status</th></tr></thead>
        <tbody>
        <?php
        if (file_exists(LEAVE_FILE)) {
           foreach (file(LEAVE_FILE, FILE_IGNORE_NEW_LINES) as $line) {
    list($leaveUser, $from, $to, $reason, $status) = explode('|', $line);
    if ($leaveUser === $user) {
        echo "<tr><td>$from</td><td>$to</td><td>$reason</td><td>$status</td></tr>";
    }
}
        }
        ?>
        </tbody>
    </table>
</body>
<script>
function toggleBreakDetails(day) {
    const row = document.getElementById('break-details-' + day);
    row.style.display = row.style.display === 'table-row' ? 'none' : 'table-row';
}
</script>
</html>
